@@ -1,4 +1,3 @@
-# coding=utf-8
 # Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
 # Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
 #
@@ -18,30 +17,28 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 from time import sleep
+from typing import Optional, Dict, Tuple, List, Callable, Iterable, Union, Any
 import re
 import os
-import signal
-import atexit
 import sys
 import errno
+from gettext import gettext as _
 import logging
 import logging.handlers
 import argparse
-from ctypes import cdll, byref, create_string_buffer
+from ctypes import cdll, byref, create_string_buffer, sizeof, c_long
 import traceback
 import fcntl
 import struct
-import termios
+import socket
+import array
+import time
+
+import cairo
 
 from blueman.main.Config import Config
-
-try:
-    in_fg = os.getpgrp() == struct.unpack(str('h'), fcntl.ioctl(0, termios.TIOCGPGRP, "  "))[0]
-except IOError:
-    in_fg = 'DEBUG' in os.environ
-
 from blueman.main.DBusProxies import AppletService, DBusProxyFailed
-from blueman.Constants import *
+from blueman.Constants import BIN_DIR, ICON_PATH
 
 import gi
 gi.require_version("Gtk", "3.0")
@@ -49,17 +46,14 @@ gi.require_version("Gdk", "3.0")
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
-from gi.repository import GLib
 from gi.repository import Gio
 
-
-__all__ = ["check_bluetooth_status", "launch", "setup_icon_path", "get_icon",
-           "get_notification_icon", "adapter_path_to_name", "e_", "opacify_pixbuf", "composite_icon",
-           "format_bytes", "create_menuitem", "get_lockfile", "get_pid", "is_running", "check_single_instance", "kill",
-           "have", "set_proc_title", "create_logger", "create_parser", "open_rfcomm"]
+__all__ = ["check_bluetooth_status", "launch", "setup_icon_path", "adapter_path_to_name", "e_", "bmexit",
+           "format_bytes", "create_menuitem", "have", "set_proc_title", "create_logger", "create_parser", "open_rfcomm",
+           "get_local_interfaces"]
 
 
-def check_bluetooth_status(message, exitfunc):
+def check_bluetooth_status(message: str, exitfunc: Callable[[], Any]) -> None:
     try:
         applet = AppletService()
     except DBusProxyFailed as e:
@@ -73,7 +67,7 @@ def check_bluetooth_status(message, exitfunc):
 
     if not applet.GetBluetoothStatus():
         d = Gtk.MessageDialog(
-            None, type=Gtk.MessageType.ERROR, icon_name="blueman",
+            type=Gtk.MessageType.ERROR, icon_name="blueman",
             text=_("Bluetooth Turned Off"), secondary_text=message)
         d.add_button(_("Exit"), Gtk.ResponseType.NO)
         d.add_button(_("Enable Bluetooth"), Gtk.ResponseType.YES)
@@ -94,7 +88,7 @@ def check_bluetooth_status(message, exitfunc):
     config = Config("org.blueman.plugins.powermanager")
     if config["auto-power-on"] is None:
         d = Gtk.MessageDialog(
-            None, type=Gtk.MessageType.QUESTION, icon_name="blueman",
+            type=Gtk.MessageType.QUESTION, icon_name="blueman",
             text=_("Shall bluetooth get enabled automatically?"))
         d.add_button(_("Yes"), Gtk.ResponseType.YES)
         d.add_button(_("No"), Gtk.ResponseType.NO)
@@ -105,12 +99,27 @@ def check_bluetooth_status(message, exitfunc):
         config["auto-power-on"] = resp == Gtk.ResponseType.YES
 
 
-def launch(cmd, paths=None, system=False, icon_name=None, sn=True, name="blueman"):
+def launch(
+    cmd: str,
+    paths: Optional[Iterable[str]] = None,
+    system: bool = False,
+    icon_name: Optional[str] = None,
+    name: str = "blueman",
+    sn: bool = True,
+) -> bool:
     """Launch a gui app with starup notification"""
-    display = Gdk.Display.get_default()
-    timestamp = Gtk.get_current_event_time()
-    context = display.get_app_launch_context()
-    context.set_timestamp(timestamp)
+    context = None
+    gtktimestamp = Gtk.get_current_event_time()
+    if gtktimestamp == 0:
+        logging.info("Gtk eventtime is 0, not using LaunchContext")
+        timestamp = int(time.clock_gettime(time.CLOCK_MONOTONIC_RAW))
+    else:
+        timestamp = gtktimestamp
+        display = Gdk.Display.get_default()
+        assert display
+        context = display.get_app_launch_context()
+        context.set_timestamp(timestamp)
+
     if sn:
         flags = Gio.AppInfoCreateFlags.SUPPORTS_STARTUP_NOTIFICATION
     else:
@@ -125,72 +134,39 @@ def launch(cmd, paths=None, system=False, icon_name=None, sn=True, name="blueman
         cmd = os.path.expanduser(cmd)
 
     if paths:
-        files = [Gio.File.new_for_commandline_arg(p) for p in paths]
+        files: Optional[List[Gio.File]] = [Gio.File.new_for_commandline_arg(p) for p in paths]
     else:
         files = None
 
-    if icon_name:
-        icon = Gio.Icon.new_for_string(icon_name)
-        context.set_icon(icon)
+    if icon_name and context is not None:
+        context.set_icon_name(icon_name)
 
     appinfo = Gio.AppInfo.create_from_commandline(cmd, name, flags)
-    launched = appinfo.launch(files, context)
+    launched: bool = appinfo.launch(files, context)
 
     if not launched:
-        logging.error("Command: %s failed" % cmd)
+        logging.error(f"Command: {cmd} failed")
 
     return launched
 
 
-def setup_icon_path():
+def setup_icon_path() -> None:
     ic = Gtk.IconTheme.get_default()
     ic.prepend_search_path(ICON_PATH)
 
 
-def get_icon(name, size=24, fallback="image-missing"):
-    ic = Gtk.IconTheme.get_default()
-
-    try:
-        icon = ic.load_icon(name, size, 0)
-    except GLib.Error:
-        if not fallback:
-            raise
-        try:
-            icon = ic.load_icon(fallback, size, 0)
-        except GLib.Error:
-            icon = ic.load_icon("image-missing", size, 0)
-
-    if icon.props.width > size:
-        new_w = size
-        new_h = int(size * (float(icon.props.width) / icon.props.height))
-        icon = icon.scale_simple(new_w, new_h, GdkPixbuf.InterpType.BILINEAR)
-
-    if icon.props.height > size:
-        new_w = int(size * (float(icon.props.height) / icon.props.width))
-        new_h = size
-        icon = icon.scale_simple(new_w, new_h, GdkPixbuf.InterpType.BILINEAR)
-
-    return icon
-
-
-def get_notification_icon(icon, main_icon="blueman"):
-    main = get_icon(main_icon, 48)
-    sub = get_icon(icon, 24)
-
-    return composite_icon(main, [(sub, 24, 24, 255)])
-
-
-def adapter_path_to_name(path):
+def adapter_path_to_name(path: Optional[str]) -> Optional[str]:
     if path is None or path == '':
-        return
+        return None
 
     match = re.search(r".*(hci[0-9]*)", path)
     if match:
         return match.group(1)
+    return None
 
 
 # format error
-def e_(msg):
+def e_(msg: Union[str, Exception]) -> Tuple[str, Optional[str]]:
     if isinstance(msg, Exception):
         return str(msg), traceback.format_exc()
     else:
@@ -198,26 +174,7 @@ def e_(msg):
         return s, None
 
 
-def opacify_pixbuf(pixbuf, alpha):
-    new = pixbuf.copy()
-    new.fill(0x00000000)
-    pixbuf.composite(new, 0, 0, pixbuf.props.width, pixbuf.props.height, 0, 0, 1, 1,
-                     GdkPixbuf.InterpType.BILINEAR, alpha)
-    return new
-
-
-# pixbuf, [(pixbuf, x, y, alpha), (pixbuf, x, y, alpha)]
-
-def composite_icon(target, sources):
-    target = target.copy()
-    for source in sources:
-        source[0].composite(target, source[1], source[2], source[0].get_width(), source[0].get_height(), source[1],
-                            source[2], 1, 1, GdkPixbuf.InterpType.NEAREST, source[3])
-
-    return target
-
-
-def format_bytes(size):
+def format_bytes(size: float) -> Tuple[float, str]:
     size = float(size)
     if size < 1024:
         ret = size
@@ -235,7 +192,12 @@ def format_bytes(size):
     return ret, suffix
 
 
-def create_menuitem(text, icon_name=None, pixbuf=None, surface=None):
+def create_menuitem(
+    text: str,
+    icon_name: Optional[str] = None,
+    pixbuf: Optional[GdkPixbuf.Pixbuf] = None,
+    surface: Optional[cairo.Surface] = None,
+) -> Gtk.ImageMenuItem:
     image = Gtk.Image(pixel_size=16)
     if icon_name:
         image.set_from_icon_name(icon_name, Gtk.IconSize.MENU)
@@ -248,95 +210,14 @@ def create_menuitem(text, icon_name=None, pixbuf=None, surface=None):
 
     item = Gtk.ImageMenuItem(label=text, image=image, use_underline=True)
     child = item.get_child()
+    assert isinstance(child, Gtk.AccelLabel)
     child.set_use_markup(True)
     item.show_all()
 
     return item
 
 
-def get_lockfile(name):
-    cachedir = GLib.get_user_cache_dir()
-    if not os.path.exists(cachedir):
-        try:
-            os.mkdir(cachedir)
-        except OSError as e:
-            if e.errno != errno.EEXIST:
-                raise
-    return os.path.join(cachedir, "%s-%s" % (name, os.getuid()))
-
-
-def get_pid(lockfile):
-    try:
-        with open(lockfile, "r") as f:
-            return int(f.readline())
-    except (ValueError, IOError):
-        pass
-
-
-def is_running(name, pid):
-    if not os.path.exists("/proc/%s" % pid):
-        return False
-
-    with open("/proc/%s/cmdline" % pid, "r") as f:
-        return name in f.readline().replace("\0", " ")
-
-
-def check_single_instance(name, unhide_func=None):
-    print("%s version %s starting" % (name, VERSION))
-    lockfile = get_lockfile(name)
-
-    def handler(signum, frame):
-        if unhide_func:
-            try:
-                with open(lockfile, "r") as f:
-                    f.readline()
-                    event_time = int(f.readline())
-            except ValueError:
-                event_time = 0
-
-            unhide_func(event_time)
-
-    signal.signal(signal.SIGUSR1, handler)
-
-    if os.path.exists(lockfile):
-        pid = get_pid(lockfile)
-        if pid:
-            if not is_running(name, pid):
-                print("Stale PID, overwriting")
-                os.remove(lockfile)
-            else:
-                print("There is an instance already running")
-                time = os.getenv("BLUEMAN_EVENT_TIME") or 0
-
-                with open(lockfile, "w") as f:
-                    f.write("%s\n%s" % (str(pid), str(time)))
-
-                os.kill(pid, signal.SIGUSR1)
-                exit()
-        else:
-            os.remove(lockfile)
-
-    try:
-        fd = os.open(lockfile, os.O_WRONLY | os.O_CREAT | os.O_EXCL, 0o664)
-        pid_str = "%s\n%s" % (str(os.getpid()), "0")
-        os.write(fd, pid_str.encode("UTF-8"))
-        os.close(fd)
-    except OSError:
-        print("There is an instance already running")
-        exit()
-
-    atexit.register(lambda: os.remove(lockfile))
-
-
-def kill(pid, name):
-    if pid and is_running(name, pid):
-        print('Terminating ' + name)
-        os.kill(pid, signal.SIGTERM)
-        return True
-    return False
-
-
-def have(t):
+def have(t: str) -> Optional[str]:
     paths = os.environ['PATH'] + ':/sbin:/usr/sbin'
     for path in paths.split(os.pathsep):
         exec_path = os.path.join(path, t)
@@ -347,7 +228,7 @@ def have(t):
     return None
 
 
-def set_proc_title(name=None):
+def set_proc_title(name: Optional[str] = None) -> int:
     """Set the process title"""
 
     if not name:
@@ -356,7 +237,7 @@ def set_proc_title(name=None):
     libc = cdll.LoadLibrary('libc.so.6')
     buff = create_string_buffer(len(name) + 1)
     buff.value = name.encode("UTF-8")
-    ret = libc.prctl(15, byref(buff), 0, 0, 0)
+    ret: int = libc.prctl(15, byref(buff), 0, 0, 0)
 
     if ret != 0:
         logging.error("Failed to set process title")
@@ -369,7 +250,13 @@ syslog_logger_format = '%(name)s %(levelname)s %(module)s:%(lineno)s %(funcName)
 logger_date_fmt = '%H.%M.%S'
 
 
-def create_logger(log_level, name, log_format=None, date_fmt=None, syslog=False):
+def create_logger(
+    log_level: int,
+    name: str,
+    log_format: Optional[str] = None,
+    date_fmt: Optional[str] = None,
+    syslog: bool = False,
+) -> logging.Logger:
     if log_format is None:
         log_format = logger_format
     if date_fmt is None:
@@ -388,7 +275,11 @@ def create_logger(log_level, name, log_format=None, date_fmt=None, syslog=False)
     return logger
 
 
-def create_parser(parser=None, syslog=True, loglevel=True):
+def create_parser(
+    parser: Optional[argparse.ArgumentParser] = None,
+    syslog: bool = True,
+    loglevel: bool = True,
+) -> argparse.ArgumentParser:
     if parser is None:
         parser = argparse.ArgumentParser()
 
@@ -401,13 +292,60 @@ def create_parser(parser=None, syslog=True, loglevel=True):
     return parser
 
 
-def open_rfcomm(file, mode):
+def open_rfcomm(file: str, mode: int) -> int:
     try:
         return os.open(file, mode | os.O_EXCL | os.O_NONBLOCK | os.O_NOCTTY)
     except OSError as err:
         if err.errno == errno.EBUSY:
-            logging.warning('%s is busy, delaying 2 seconds' % file)
+            logging.warning(f"{file} is busy, delaying 2 seconds")
             sleep(2)
             return open_rfcomm(file, mode)
         else:
             raise
+
+
+def _netmask_for_ifacename(name: str, sock: socket.socket) -> Optional[str]:
+    siocgifnetmask = 0x891b
+    bytebuf = struct.pack('256s', name.encode('utf-8'))
+    try:
+        ret = fcntl.ioctl(sock.fileno(), siocgifnetmask, bytebuf)
+    except OSError:
+        logging.error('siocgifnetmask failed')
+        return None
+
+    return socket.inet_ntoa(ret[20:24])
+
+
+def get_local_interfaces() -> Dict[str, Tuple[str, Optional[str]]]:
+    """ Returns a dictionary of name:ip, mask key value pairs. """
+    siocgifconf = 0x8912
+    names = array.array('B', 4096 * b'\0')
+    names_address, names_length = names.buffer_info()
+    mutable_byte_buffer = struct.pack('iL', 4096, names_address)
+
+    try:
+        with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
+            try:
+                mutated_byte_buffer = fcntl.ioctl(sock.fileno(), siocgifconf, mutable_byte_buffer)
+            except OSError:
+                logging.error('siocgifconf failed')
+                return {}
+
+            max_bytes_out, names_address_out = struct.unpack('iL', mutated_byte_buffer)
+            namestr = names.tobytes()
+
+            ip_dict = {}
+            for i in range(0, max_bytes_out, 24 + 2 * sizeof(c_long)):
+                name = namestr[i: i + 16].split(b'\0', 1)[0].decode('utf-8')
+                ipaddr = socket.inet_ntoa(namestr[i + 20: i + 24])
+                mask = _netmask_for_ifacename(name, sock)
+                ip_dict[name] = (ipaddr, mask)
+    except OSError:
+        logging.error('Socket creation failed', exc_info=True)
+        return {}
+
+    return ip_dict
+
+
+def bmexit(msg: Optional[Union[str, int]] = None) -> None:
+    raise SystemExit(msg)
