@@ -413,8 +413,7 @@ class ManagerDeviceList(DeviceList):
         # cinfo init may fail for bluetooth devices version 4 and up
         # FIXME Workaround is horrible and we should show something better
         if cinfo.failed:
-            if not bars:
-                bars = {"rssi": 100.0, "tpl": 100.0, "lq": 100.0}
+            bars.update({"rssi": 100.0, "tpl": 100.0, "lq": 100.0})
         else:
             try:
                 bars["rssi"] = max(50 + float(cinfo.get_rssi()) / 127 * 50, 10)
@@ -446,9 +445,9 @@ class ManagerDeviceList(DeviceList):
         if row["battery"] == row["rssi"] == row["tpl"] == row["lq"] == 0:
             return
 
-        self.set(tree_iter, rssi=0, lq=0, tpl=0)
-        self._prepare_fader(row["cell_fader"], lambda: self.set(tree_iter, rssi_pb=None, lq_pb=None, tpl_pb=None))\
-            .animate(start=1.0, end=0.0, duration=400)
+        self.set(tree_iter, battery=0, rssi=0, lq=0, tpl=0)
+        self._prepare_fader(row["cell_fader"], lambda: self.set(tree_iter, battery_pb=None, rssi_pb=None, lq_pb=None,
+                                                                tpl_pb=None)).animate(start=1.0, end=0.0, duration=400)
 
     def _prepare_fader(self, fader: AnimBase, callback: Optional[Callable[[], None]] = None) -> AnimBase:
         def on_finished(finished_fader: AnimBase) -> None:
